package com.sillycycle.bagleyd.panex.model;

/*
 * @(#)PanexFormat.java
 *
 * Copyright 2014 - 2022  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import com.sillycycle.bagleyd.util.Point;

/**
 * The <code>PanexFormat</code> class holds all abstract info about
 * the panex widget.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/panex.html
 */

public final class PanexFormat {
	static final boolean DEBUG = false;
	public enum Modes {HANOI, ALGORITHME, PANEX}
	public static final Modes defaultMode = Modes.PANEX;
	public static final int MAX_COLUMNS = 3;
	public static final int MIN_TILES = 1;
	//public static final int DEFAULT_TILES = 10;
	public static final int DEFAULT_TILES = 3;
	public static final int BASE = 10;

	PanexColumn[] columns;
	Point[][] positionOfTile;
	private int[] startPositions;
	private int[] finalPositions;
	int mode; // 0-2
	int colors; // 0-1
	int tiles; // > 0
	int blanks;
	int slots;

	public PanexFormat(int mode, int tiles) {
		this.mode = mode;
		this.tiles = tiles;
		resetFormat(true);
	}

	public final void resetFormat(boolean start) {
		columns = new PanexColumn[MAX_COLUMNS];
		startPositions = new int[MAX_COLUMNS];
		finalPositions = new int[MAX_COLUMNS];
		if (mode == Modes.HANOI.ordinal()) {
			colors = 1;
			blanks = 0;
			slots = tiles + blanks;
			startPositions[0] = 0;
			startPositions[1] = -1;
			startPositions[2] = -1;
			finalPositions[0] = -1;
			finalPositions[1] = -1;
			finalPositions[2] = 0;
			columns[0] = new PanexColumn(tiles, blanks, 0);
			columns[1] = new PanexColumn(0, slots, 1);
			columns[2] = new PanexColumn(0, slots, 2);
		} else if (mode == Modes.ALGORITHME.ordinal()) {
			colors = 2;
			blanks = ((tiles == 1) ? 0 : tiles - 2);
			slots = tiles + blanks;
			startPositions[0] = 0;
			startPositions[1] = -1;
			startPositions[2] = 1;
			finalPositions[0] = 1;
			finalPositions[1] = -1;
			finalPositions[2] = 0;
			columns[0] = new PanexColumn(tiles, blanks, 0);
			columns[1] = new PanexColumn(0, slots, 1);
			columns[2] = new PanexColumn(tiles, blanks, 2);
		} else if (mode == Modes.PANEX.ordinal()) {
			colors = 2;
			blanks = 1;
			slots = tiles + blanks;
			startPositions[0] = 0;
			startPositions[1] = -1;
			startPositions[2] = 1;
			finalPositions[0] = 1;
			finalPositions[1] = -1;
			finalPositions[2] = 0;
			columns[0] = new PanexColumn(tiles, blanks, 0);
			columns[1] = new PanexColumn(0, slots, 1);
			columns[2] = new PanexColumn(tiles, blanks, 2);
		}
		positionOfTile = new Point[colors][tiles];
		if (start)
			setStartPosition();
		else
			setFinalPosition();
	}

	public void checkFormat() {
		if (tiles < MIN_TILES) {
			System.out.println(tiles +
				" number of tiles in a stack incorrect, must be at least " +
				MIN_TILES +
				" defaulting to " +
				DEFAULT_TILES);
			tiles = PanexFormat.DEFAULT_TILES;
			// must reset FIXME
		}
		if (mode < Modes.HANOI.ordinal() ||
				mode > Modes.PANEX.ordinal()) {
			System.out.println(mode +
				" for mode is incorrect, use " +
				Modes.HANOI.ordinal() +
				" for " +
				Modes.HANOI.toString() +
				", " +
				Modes.ALGORITHME.ordinal() +
				" for " +
				Modes.ALGORITHME.toString() +
				", " +
				Modes.PANEX.ordinal() +
				" for " +
				Modes.PANEX.toString() +
				", defaulting to " +
				defaultMode.ordinal());
			mode = defaultMode.ordinal();
			// must reset FIXME
		}
	}

	public static String titleName(String name) {
		if (name.length() > 1) {
			return name.substring(0, 1).toUpperCase() + name.substring(1).toLowerCase();
		}
		return name.toUpperCase();
	}

	protected String paramString() {
		return "mode:" + mode +
			", tiles:" + tiles;
	}

	public int getMode() {
		return this.mode;
	}

	public void setMode(int value) {
		this.mode = value;
	}

	public int getTiles() {
		return tiles;
	}

	public void setTiles(int value) {
		tiles = value;
	}

	public int getBlanks() {
		return blanks;
	}

	public int getSlots() {
		return slots;
	}

	public int getColors() {
		return colors;
	}

	public void setColors(int value) {
		colors = value;
	}

	public PanexColumn getColumn(int column) {
		return columns[column];
	}

	public Integer getStartPosition(int column) {
		return startPositions[column];
	}

	public Integer getFinalPosition(int column) {
		return finalPositions[column];
	}

	public int getColumnOfTile(int color, int tile) {
		return positionOfTile[color][tile].x;
	}

	public int getDepthOfTile(int color, int tile) {
		return positionOfTile[color][tile].y;
	}

	public void setColumnOfTile(int color, int tile, int column) {
		if (color >= 0 && tile >= 0)
			positionOfTile[color][tile].x = column;
	}

	public void setDepthOfTile(int color, int tile, int depth) {
		if (color >= 0 && tile >= 0)
			positionOfTile[color][tile].y = depth;
	}

	public void setStartPosition() {
		for (int column = 0; column < MAX_COLUMNS; column++) {
			columns[column].setColumn(blanks, startPositions[column]);
		}
		for (int color = 0; color < colors; color++) {
			for (int tile = 0; tile < tiles; tile++) {
				positionOfTile[color][tile] = new Point();
				if (color == 0) {
					positionOfTile[color][tile].x = 0;
				} else if (color == 1) {
					positionOfTile[color][tile].x = MAX_COLUMNS - 1;
				}
				positionOfTile[color][tile].y = tile + blanks;
			}
		}
	}

	public void setFinalPosition() {
		for (int column = 0; column < MAX_COLUMNS; column++) {
			columns[column].setColumn(blanks, finalPositions[column]);
		}
		for (int color = 0; color < colors; color++) {
			for (int tile = 0; tile < tiles; tile++) {
				positionOfTile[color][tile] = new Point();
				if (color == 1) {
					positionOfTile[color][tile].x = 0;
				} else if (color == 0) {
					positionOfTile[color][tile].x = MAX_COLUMNS - 1;
				}
				positionOfTile[color][tile].y = tile + blanks;
			}
		}
	}

	public void printStacks() {
		//System.out.println("which tiles are in each stack");
		for (int position = 0; position < getSlots(); position++)
			for (int stack = 0; stack < PanexFormat.MAX_COLUMNS; stack++) {
				int color = getColumn(stack).getTileAtPosition(position).getColor();
				if (color < BASE)
					System.out.print(" ");
				System.out.print(color);
				int number = getColumn(stack).getTileAtPosition(position).getNumber();
				if (number < BASE)
					System.out.print(" ");
				System.out.print(", " + number);
				if (stack == PanexFormat.MAX_COLUMNS - 1)
					System.out.println();
				else
					System.out.print(" | ");
			}
	}

	public void printTiles() {
		//System.out.println("which stack are the tiles in");
		for (int position = 0; position < getTiles(); position++)
			for (int stack = 0; stack < getColors(); stack++) {
				int column = getColumnOfTile(stack, position);
				if (column < BASE)
					System.out.print(" ");
				System.out.print(column);
				int location = getDepthOfTile(stack, position);
				if (location < BASE)
					System.out.print(" ");
				System.out.print(", " + location);
				if (stack == getColors() - 1)
					System.out.println();
				else
					System.out.print(" | ");
			}
	}
}
